@@ -5,16 +5,16 @@ use strict ;
 use warnings;
 use bytes;
 
-use Compress::Raw::Zlib  2.033 ;
-use IO::Compress::Base::Common  2.033 qw(:Status createSelfTiedObject);
+use Compress::Raw::Zlib  2.037 ;
+use IO::Compress::Base::Common  2.037 qw(:Status createSelfTiedObject);
 
-use IO::Uncompress::Base  2.033 ;
-use IO::Uncompress::Adapter::Inflate  2.033 ;
+use IO::Uncompress::Base  2.037 ;
+use IO::Uncompress::Adapter::Inflate  2.037 ;
 
 require Exporter ;
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, %DEFLATE_CONSTANTS, $RawInflateError);
 
-$VERSION = '2.033';
+$VERSION = '2.037';
 $RawInflateError = '';
 
 @ISA    = qw( Exporter IO::Uncompress::Base );
@@ -979,7 +979,7 @@ If the C<$z> object is associated with a file or a filehandle, C<fileno>
 will return the underlying file descriptor. Once the C<close> method is
 called C<fileno> will return C<undef>.
 
-If the C<$z> object is is associated with a buffer, this method will return
+If the C<$z> object is associated with a buffer, this method will return
 C<undef>.
 
 =head2 close
