@@ -6,7 +6,7 @@ use strict;
 
 our($VERSION, @ISA, @EXPORT_OK);
 
-$VERSION = "1.17";
+$VERSION = "1.20";
 
 use Carp;
 use Exporter ();
@@ -308,7 +308,7 @@ invert_opset function.
 
     rv2sv sassign
 
-    rv2av aassign aelem aelemfast aslice av2arylen
+    rv2av aassign aelem aelemfast aelemfast_lex aslice av2arylen
 
     rv2hv helem hslice each values keys exists delete aeach akeys avalues
     boolkeys reach rvalues rkeys
@@ -335,7 +335,7 @@ invert_opset function.
 
     warn die lineseq nextstate scope enter leave
 
-    rv2cv anoncode prototype
+    rv2cv anoncode prototype coreargs
 
     entersub leavesub leavesublv return method method_named -- XXX loops via recursion?
 
