@@ -2,7 +2,7 @@
 #
 # regen.pl - a wrapper that runs all *.pl scripts to to autogenerate files
 
-require 5.003;	# keep this compatible, an old perl is all we may have before
+require 5.004;	# keep this compatible, an old perl is all we may have before
                 # we build the new one
 
 # The idea is to move the regen_headers target out of the Makefile so that
@@ -13,11 +13,10 @@ require 5.003;	# keep this compatible, an old perl is all we may have before
 
 use strict;
 
-# Which scripts to run. Note the ordering: embed.pl must run after
-# opcode.pl, since it depends on pp.sym
+# Which scripts to run.
 
 my @scripts = qw(
-keywords.pl
+mg_vtable.pl
 opcode.pl
 overload.pl
 reentr.pl
@@ -26,8 +25,9 @@ warnings.pl
 embed.pl
 );
 
-foreach my $pl (@scripts) {
+my $tap = $ARGV[0] && $ARGV[0] eq '--tap' ? '# ' : '';
+foreach my $pl (map {"regen/$_"} @scripts) {
   my @command =  ($^X, $pl, @ARGV);
-  print "@command\n";
+  print "$tap@command\n";
   system @command;
 }
