@@ -1,20 +1,20 @@
 #!\\bin\\sh
-PERL_CONFIG_SH='true'
+Author=''
 _a='.a'
 _o='.o'
 afs='false'
 afsroot='/afs'
 alignbytes='4'
-apiversion='5.005'
 aphostname='localhost'
+apiversion='5.005'
 ar=':'
 archlib='\\system\\libs\\perl\\x.y.z\\thumb-symbian'
 archlibexp='\\system\\libs\\perl\\x.y.z\\thumb-symbian'
 archname='thumb-symbian'
 asctime_r_proto='0'
 bin='\\system\\apps\\perl'
-binexp='\\system\\apps\\perl'
 bincompat5005='n'
+binexp='\\system\\apps\\perl'
 byteorder='1234'
 castflags='0'
 cc='gcc'
@@ -46,6 +46,7 @@ d_PRIo64='undef'
 d_PRIu64='undef'
 d_PRIx64='undef'
 d_SCNfldbl='undef'
+d__fwalk='undef'
 d_access='undef'
 d_accessx='undef'
 d_aintl='undef'
@@ -65,9 +66,9 @@ d_attribute_unused='undef'
 d_attribute_warn_unused_result='undef'
 d_bcmp='undef'
 d_bcopy='undef'
+d_bsd='undef'
 d_bsdgetpgrp='undef'
 d_bsdsetpgrp='undef'
-d_bsd='undef'
 d_builtin_choose_expr='undef'
 d_builtin_expect='undef'
 d_bzero='undef'
@@ -85,11 +86,11 @@ d_cmsghdr_s='undef'
 d_const='define'
 d_copysignl='undef'
 d_cplusplus='undef'
-d_crypt_r='undef'
 d_crypt='undef'
+d_crypt_r='undef'
 d_csh='undef'
-d_ctermid_r='undef'
 d_ctermid='undef'
+d_ctermid_r='undef'
 d_ctime64='undef'
 d_ctime_r='undef'
 d_cuserid='undef'
@@ -104,20 +105,20 @@ d_dlerror='undef'
 d_dlopen='undef'
 d_dlsymun='undef'
 d_dosuid='undef'
-d_drand48proto='undef'
 d_drand48_r='undef'
+d_drand48proto='undef'
 d_dup2='undef'
 d_eaccess='undef'
-d_endgrent_r='undef'
 d_endgrent='undef'
+d_endgrent_r='undef'
 d_endhent='undef'
 d_endhostent_r='undef'
 d_endnent='undef'
 d_endnetent_r='undef'
 d_endpent='undef'
 d_endprotoent_r='undef'
-d_endpwent_r='undef'
 d_endpwent='undef'
+d_endpwent_r='undef'
 d_endsent='undef'
 d_endservent_r='undef'
 d_eofnblk='undef'
@@ -126,23 +127,23 @@ d_faststdio='undef'
 d_fchdir='undef'
 d_fchmod='undef'
 d_fchown='undef'
-d_fcntl_can_lock='undef'
 d_fcntl='undef'
+d_fcntl_can_lock='undef'
 d_fd_macros='undef'
-d_fds_bits='undef'
 d_fd_set='undef'
+d_fds_bits='undef'
 d_fgetpos='undef'
-d_finitel='undef'
 d_finite='undef'
+d_finitel='undef'
 d_flexfnam='define'
-d_flockproto='undef'
 d_flock='undef'
+d_flockproto='undef'
 d_fork='undef'
+d_fp_class='undef'
 d_fpathconf='undef'
+d_fpclass='undef'
 d_fpclassify='undef'
 d_fpclassl='undef'
-d_fp_class='undef'
-d_fpclass='undef'
 d_fpos64_t='undef'
 d_frexpl='undef'
 d_fs_data_s='undef'
@@ -153,17 +154,16 @@ d_fstatvfs='undef'
 d_fsync='undef'
 d_ftello='undef'
 d_ftime='undef'
-d_futimesat='undef'
 d_futimes='undef'
-d__fwalk='undef'
+d_futimesat='undef'
 d_gdbm_ndbm_h_uses_prototypes='undef'
 d_gdbmndbm_h_uses_prototypes='undef'
 d_getaddrinfo='undef'
 d_getcwd='define'
 d_getespwnam='undef'
 d_getfsstat='undef'
-d_getgrent_r='undef'
 d_getgrent='undef'
+d_getgrent_r='undef'
 d_getgrgid_r='undef'
 d_getgrnam_r='undef'
 d_getgrps='undef'
@@ -176,10 +176,10 @@ d_gethostbyname_r='undef'
 d_gethostent_r='undef'
 d_gethostprotos='define'
 d_getitimer='undef'
-d_getlogin_r='undef'
 d_getlogin='undef'
-d_getmntent='undef'
+d_getlogin_r='undef'
 d_getmnt='undef'
+d_getmntent='undef'
 d_getnameinfo='undef'
 d_getnbyaddr='undef'
 d_getnbyname='undef'
@@ -202,8 +202,8 @@ d_getprotobynumber_r='undef'
 d_getprotoent_r='undef'
 d_getprotoprotos='define'
 d_getprpwnam='undef'
-d_getpwent_r='undef'
 d_getpwent='undef'
+d_getpwent_r='undef'
 d_getpwnam_r='undef'
 d_getpwuid_r='undef'
 d_getsbyname='define'
@@ -214,8 +214,8 @@ d_getservbyport_r='undef'
 d_getservent_r='undef'
 d_getservprotos='define'
 d_getspent='undef'
-d_getspnam_r='undef'
 d_getspnam='undef'
+d_getspnam_r='undef'
 d_gettimeod='define'
 d_gmtime64='undef'
 d_gmtime_r='undef'
@@ -233,16 +233,16 @@ d_int64_t='undef'
 d_isascii='undef'
 d_isfinite='undef'
 d_isinf='undef'
-d_isnanl='undef'
 d_isnan='undef'
+d_isnanl='undef'
 d_killpg='undef'
 d_lchown='undef'
 d_ldbl_dig='undef'
 d_libm_lib_version='undef'
 d_link='undef'
 d_localtime64='undef'
-d_localtime_r_needs_tzset='undef'
 d_localtime_r='undef'
+d_localtime_r_needs_tzset='undef'
 d_locconv='undef'
 d_lockf='undef'
 d_longdbl='undef'
@@ -263,26 +263,26 @@ d_memset='define'
 d_mkdir='define'
 d_mkdtemp='undef'
 d_mkfifo='undef'
-d_mkstemps='undef'
 d_mkstemp='undef'
+d_mkstemps='undef'
 d_mktime64='undef'
 d_mktime='undef'
 d_mmap='undef'
+d_modfl='undef'
 d_modfl_pow32_bug='undef'
 d_modflproto='undef'
-d_modfl='undef'
 d_mprotect='undef'
-d_msgctl='undef'
+d_msg='undef'
 d_msg_ctrunc='undef'
 d_msg_dontroute='undef'
-d_msgget='undef'
-d_msghdr_s='undef'
 d_msg_oob='undef'
 d_msg_peek='undef'
 d_msg_proxy='undef'
+d_msgctl='undef'
+d_msgget='undef'
+d_msghdr_s='undef'
 d_msgrcv='undef'
 d_msgsnd='undef'
-d_msg='undef'
 d_msync='undef'
 d_munmap='undef'
 d_mymalloc='undef'
@@ -393,6 +393,7 @@ d_sigsetjmp='undef'
 d_sin6_scope_id='undef'
 d_sitearch='define'
 d_sitecustomize='undef'
+d_sitecustomize='undef'
 d_snprintf='undef'
 d_sockaddr_sa_len='undef'
 d_sockatmark='undef'
@@ -409,8 +410,8 @@ d_sresgproto='undef'
 d_sresuproto='undef'
 d_statblks='undef'
 d_statfs_f_flags='undef'
-d_static_inline='undef'
 d_statfs_s='undef'
+d_static_inline='undef'
 d_statvfs='undef'
 d_stdio_cnt_lval='undef'
 d_stdio_ptr_lval='undef'
@@ -465,7 +466,6 @@ d_uname='undef'
 d_union_semun='undef'
 d_unordered='undef'
 d_unsetenv='undef'
-d_sitecustomize='undef'
 d_usleep='define'
 d_usleepproto='undef'
 d_ustat='undef'
@@ -597,6 +597,7 @@ i_sgtty='undef'
 i_shadow='undef'
 i_socks='undef'
 i_stdarg='define'
+i_stdbool='undef'
 i_stddef='undef'
 i_stdlib='define'
 i_string='define'
@@ -653,10 +654,10 @@ intsize='4'
 ivdformat='"ld"'
 ivsize='4'
 ivtype='long'
-lib_ext='.a'
-lddlflags=''
 ld=':'
+lddlflags=''
 ldflags=''
+lib_ext='.a'
 libc='stdlib'
 libm_lib_version='0'
 libperl='libperl.a'
@@ -682,11 +683,11 @@ netdb_host_type='const char *'
 netdb_name_type='const char *'
 netdb_net_type='unsigned long'
 nroff='nroff'
+nv_overflows_integers_at='256.0*256.0*256.0*256.0*256.0*256.0*2.0*2.0*2.0*2.0*2.0'
 nv_preserves_uv_bits='0'
 nveformat='"e"'
 nvfformat='"f"'
 nvgformat='"g"'
-nv_overflows_integers_at='256.0*256.0*256.0*256.0*256.0*256.0*2.0*2.0*2.0*2.0*2.0'
 nvsize='8'
 nvtype='double'
 o_nonblock='O_NONBLOCK'
@@ -698,7 +699,7 @@ osname='symbian'
 osvers='7.0s'
 otherlibdirs=''
 package=''
-path_sep=';';
+path_sep=';'
 perl_static_inline='static'
 phostname='hostname'
 pidtype='int'
@@ -749,8 +750,8 @@ setnetent_r_proto='0'
 setprotoent_r_proto='0'
 setpwent_r_proto='0'
 setservent_r_proto='0'
-shmattype='void *'
 sh=':'
+shmattype='void *'
 shortsize=2
 sig_name_init='0'
 sig_num_init='0'
@@ -771,6 +772,8 @@ srand48_r_proto='0'
 srandom_r_proto='0'
 src=''
 ssizetype=int
+st_ino_sign='1'
+st_ino_size='4'
 startperl=''
 stdchar=char
 stdio_base='((fp)->_IO_read_base)'
@@ -779,8 +782,6 @@ stdio_cnt='((fp)->_IO_read_end - (fp)->_IO_read_ptr)'
 stdio_filbuf=''
 stdio_ptr='((fp)->_IO_read_ptr)'
 stdio_stream_array=''
-st_ino_sign='1'
-st_ino_size='4'
 strerror_r_proto='0'
 targetarch='thumb-symbian'
 timetype=time_t
@@ -809,6 +810,7 @@ usedl='undef'
 usedtrace='undef'
 usefaststdio='undef'
 useithreads='undef'
+usekernprocpathname='undef'
 uselargefiles='undef'
 uselongdouble='undef'
 usemallocwrap='define'
@@ -816,6 +818,7 @@ usemorebits='undef'
 usemultiplicity='undef'
 usemymalloc='n'
 usenm='false'
+usensgetexecutablepath='undef'
 useopcode='true'
 useperlio='define'
 useposix='true'
@@ -833,16 +836,18 @@ uvoformat='"lo"'
 uvsize='4'
 uvtype='unsigned long'
 uvuformat='"lu"'
+uvxformat='"lx"'
 vaproto='undef'
-vendorlib_stem=''
-vendorlib=''
-vendorlibexp=''
 vendorarch=''
 vendorarchexp=''
+vendorlib=''
+vendorlib_stem=''
+vendorlibexp=''
 vendorprefix=''
 vendorprefixexp=''
 version='x.y.z'
-uvxformat='"lx"'
 versiononly='undef'
 voidflags=1
 xs_apiversion='5.008'
+zip=''
+PERL_CONFIG_SH='true'
